/*
 * unboundbibleimporter.cpp
 *
 * A class to import Unbound Bible files into Emdros
 * Created: 12/16-2005
 * Last update: 8/18-2006 
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2005-2006  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation, license version 2.  
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *   02111-1307 USA
 *
 *
 *   Special exception
 *   =================
 * 
 *   In addition, as a special exception, Ulrik Petersen, the
 *   copyright holder of Emdros, gives permission to link Emdros, in
 *   whole or in part, with the libraries which are normally
 *   distributed with:
 *   
 *   a) Sun's Java platform,
 *   b) Python, 
 *   c) Jython,
 *   d) Ruby, and/or 
 *   e) Perl 
 *   f) PostgreSQL
 *   g) OpenSSL
 *
 *   (or with modified versions of these), and to distribute linked
 *   combinations including both Emdros, in whole or in part, and one
 *   or more of the libraries normally distributed with (a)-(g) above.
 *
 *   Please note: This gives you special rights concerning the
 *   libraries which normally accompany the above pieces of software.
 *   It gives you no special rights concerning software that you write
 *   yourself.  You must obey the GNU General Public License in all
 *   respects for all of the code used other than the libraries
 *   normally distributed with (a)-(g) above.
 *
 *   If you modify this file, you may extend this exception to your
 *   version of the file, but you are not obligated to do so. If you
 *   do not wish to do so, delete this exception statement from your
 *   version.
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 *
 *   Website
 *   =======
 *
 *   Emdros has a website here:
 *
 *   http://emdros.org
 *
 *
 *
 **************************************************************************/

#ifndef _UNBOUNDBIBLE_IMPORTER_H__
#define _UNBOUNDBIBLE_IMPORTER_H__

#include <map>
#include <string>
#include <iostream>

#include <emdf_output.h>
#include <list>
#include <infos.h>

typedef std::map<std::string, std::string> strstrmap;

class SetOfMonads; // Forward declaration

class EmdrosMemObject {
	SetOfMonads m_monads;
	strstrmap m_features;
	id_d_t m_id_d;
public:
	EmdrosMemObject();
	EmdrosMemObject(const SetOfMonads& monads, id_d_t id_d = NIL);
	EmdrosMemObject(monad_m monad, id_d_t id_d = NIL);
	EmdrosMemObject(monad_m first_m, monad_m last_m, id_d_t id_d = NIL);
	~EmdrosMemObject() {};
	void addMonads(const SetOfMonads& monads); // Does a union-with on m_monads
	void setMonads(const SetOfMonads& monads); // Does an assignment to m_monads
	const SetOfMonads& getMonads(void) const;
	const std::string& getFeature(const std::string& feature_name) const; // Returns empty string if not there...
	void setFeature(const std::string& feature_name, const std::string& value);
	bool putMQL(std::ostream *pOut, const std::list<FeatureInfo>& feature_infos, const std::string& object_type_name = "") const;
	bool hasFeature(const std::string& feature_name) const;
};

typedef std::list<EmdrosMemObject> EMOList;

class UnboundBibleImporter {
 protected:
	EMOList m_books;
	EMOList m_chapters;
	EMOList m_verses;
	EMOList m_words;
	std::string m_last_book;
	long m_last_chapter;
	long m_last_verse;
	monad_m m_curmonad;
 public:
	UnboundBibleImporter() { };
	~UnboundBibleImporter() {};
	bool readBibleFromStream(std::istream *pIn);
	void putSchema(std::ostream *pOut) const;
	void putBibleInMQL(std::ostream *pOut) const;
 protected:
	void add_bcv(const std::string& book, long chapter, long verse);
	void add_word(const std::string& surface);
	EmdrosMemObject createBook(const std::string& bookname);
	EmdrosMemObject createChapter(const std::string& bookname, long chapter);
	EmdrosMemObject createVerse(const std::string& bookname, long chapter, long verse);
	void closeLastObject(EMOList& list);
};


#endif
